@@ -822,7 +822,7 @@ set_members_in_gw_item (EGwItem  *item, EContact *contact, EBookBackendGroupwise
 	e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, count);
 	items = NULL;
 	if (count)
-		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "members", filter, &items);
+		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "name email default members", filter, &items);
 	for (; items != NULL; items = g_list_next (items )) {
 		GList *emails;
 		GList *ptr;
@@ -1126,6 +1126,10 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 			e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 			return;
 		}
+		if (!egwb->priv->is_writable) {
+			e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_PermissionDenied, NULL);
+			return;
+		}
 		contact = e_contact_new_from_vcard(vcard);
 		item = e_gw_item_new_empty ();
 		e_gw_item_set_item_type (item, e_contact_get (contact, E_CONTACT_IS_LIST) ? E_GW_ITEM_TYPE_GROUP :E_GW_ITEM_TYPE_CONTACT);
@@ -1158,7 +1162,9 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 		status = e_gw_connection_create_item (egwb->priv->cnc, item, &id);  
 		if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION) 
 			status = e_gw_connection_create_item (egwb->priv->cnc, item, &id);  
-		if (status == E_GW_CONNECTION_STATUS_OK) {
+
+		/* Make sure server has returned  an id for the created contact */
+		if (status == E_GW_CONNECTION_STATUS_OK && id) {
 			e_contact_set (contact, E_CONTACT_UID, id);
 			g_free (id);
 			e_book_backend_cache_add_contact (egwb->priv->cache, contact);
@@ -1199,6 +1205,11 @@ e_book_backend_groupwise_remove_contacts (EBookBackend *backend,
 			return;
 		}
 		
+		if (!ebgw->priv->is_writable) {
+			e_data_book_respond_remove_contacts (book, opid, GNOME_Evolution_Addressbook_PermissionDenied, NULL);
+			return;
+		}
+		
 		for ( ; id_list != NULL; id_list = g_list_next (id_list)) {
 			id = (char*) id_list->data;
 			e_gw_connection_remove_item (ebgw->priv->cnc, ebgw->priv->container_id, id);
@@ -1287,7 +1298,10 @@ e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 			e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 			return;
 		}
-
+		if (!egwb->priv->is_writable) {
+			e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_PermissionDenied, NULL);
+			return;
+		}
 		contact = e_contact_new_from_vcard(vcard);
 		new_item = e_gw_item_new_empty ();
 
@@ -1835,9 +1849,9 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 			
 			ids = e_book_backend_cache_search (egwb->priv->cache, query);
 			if (ids->len > 0) {
-				status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, "members", ids, &gw_items);
+				status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, "name email default members", ids, &gw_items);
 				if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
-				status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, "members", ids, &gw_items);
+				status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, "name email default members", ids, &gw_items);
 			}
 			match_needed = FALSE;
 			g_ptr_array_free (ids, TRUE);
@@ -1848,9 +1862,9 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 				filter = e_book_backend_groupwise_build_gw_filter (egwb, query, &is_auto_completion, NULL);
 			if (filter)
 				match_needed = FALSE;
-			status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "members", filter, &gw_items);
+			status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "name email default members", filter, &gw_items);
 			if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
-				status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "members", filter, &gw_items);
+				status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "name email default members", filter, &gw_items);
 			
 		}
 		
@@ -1941,8 +1955,7 @@ book_view_thread (gpointer data)
 	gwb  = closure->bg;
 	gw_items = NULL;
 
-
-	bonobo_object_ref (book_view);
+       	bonobo_object_ref (book_view);
 	g_mutex_lock (closure->mutex);
 	g_cond_signal (closure->cond);
 	g_mutex_unlock (closure->mutex);
@@ -1985,7 +1998,7 @@ book_view_thread (gpointer data)
 		}
 
 		filter = e_book_backend_groupwise_build_gw_filter (gwb, query, &is_auto_completion, &search_string);
-		view = "members";
+		view = "name email default members";
 		if (is_auto_completion) 
 			view = "name email";
 		if (is_auto_completion && !gwb->priv->is_writable) {
@@ -2119,7 +2132,7 @@ e_book_backend_groupwise_get_changes (EBookBackend *backend,
 
 
 #define CURSOR_ITEM_LIMIT 100
-
+/*
 static gpointer
 build_cache (EBookBackendGroupwise *ebgw)
 {
@@ -2131,7 +2144,7 @@ build_cache (EBookBackendGroupwise *ebgw)
 	EBookBackendGroupwisePrivate *priv = ebgw->priv;
 	
 	
-	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "members", NULL, &gw_items);
+	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "name email default members", NULL, &gw_items);
 	if (status != E_GW_CONNECTION_STATUS_OK) 
 		return FALSE;
 	
@@ -2153,35 +2166,36 @@ build_cache (EBookBackendGroupwise *ebgw)
 	
 	
 	return NULL;
-}
+}*/
 
 /*FIXME using cursors for address book seems to be crashing server 
 till it gets fixed we will use get items. cursor implementation is below */
-/*
+
 static gpointer
 build_cache (EBookBackendGroupwise *ebgw)
 {
 	int status;
-	GList *gw_items = NULL;
+	GList *gw_items = NULL, *l;
 	EContact *contact;
 	int cursor;
 	gboolean done = FALSE;
 	EBookBackendGroupwisePrivate *priv = ebgw->priv;
+	const char *position = E_GW_CURSOR_POSITION_START;
 	
 	
-	status = e_gw_connection_create_cursor (priv->cnc, priv->container_id, "members", NULL, &cursor);
+	status = e_gw_connection_create_cursor (priv->cnc, priv->container_id, "name email default members", NULL, &cursor);
 	if (status != E_GW_CONNECTION_STATUS_OK) 
 		return FALSE;
 	while (!done) {
 
-		status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, FALSE, CURSOR_ITEM_LIMIT, &gw_items);
+		status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, TRUE, CURSOR_ITEM_LIMIT, position, &gw_items);
 
-		for (; gw_items != NULL; gw_items = g_list_next(gw_items)) { 
+		for (l = gw_items; l != NULL; l = g_list_next (l)) { 
 			contact = e_contact_new ();
-			fill_contact_from_gw_item (contact, E_GW_ITEM (gw_items->data), ebgw->priv->categories_by_id);
+			fill_contact_from_gw_item (contact, E_GW_ITEM (l->data), ebgw->priv->categories_by_id);
 			e_book_backend_cache_add_contact (ebgw->priv->cache, contact);
 			g_object_unref(contact);
-			g_object_unref (gw_items->data);
+			g_object_unref (l->data);
 			
 		}
 		if (!gw_items  || g_list_length (gw_items) == 0) {
@@ -2191,11 +2205,13 @@ build_cache (EBookBackendGroupwise *ebgw)
 		}
 	       	
 		g_list_free (gw_items);
+		gw_items = NULL;
+		position = E_GW_CURSOR_POSITION_CURRENT;
 	}
 	
 	e_gw_connection_destroy_cursor (priv->cnc, priv->container_id, cursor);
 	return NULL;
-}*/
+}
 
 static gboolean
 update_cache (EBookBackendGroupwise *ebgw)
@@ -2219,7 +2235,7 @@ update_cache (EBookBackendGroupwise *ebgw)
 
 	filter = e_gw_filter_new ();
 	e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_GREATERTHAN, "modified", time_string);
-	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "members", filter, &gw_items);
+	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "name email default members", filter, &gw_items);
 	if (status != E_GW_CONNECTION_STATUS_OK)
 		return FALSE;
 	
@@ -2316,7 +2332,7 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 			e_book_backend_notify_writable (backend, is_writable);
 			e_book_backend_notify_connection_status (backend, TRUE); 
 			priv->is_writable = is_writable;
-			e_gw_connection_get_categories (priv->cnc, priv->categories_by_id, priv->categories_by_name);
+			e_gw_connection_get_categories (priv->cnc, &priv->categories_by_id, &priv->categories_by_name);
 			if (!e_gw_connection_get_version(priv->cnc))  
 				e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_InvalidServerVersion);
 			else
@@ -2595,15 +2611,6 @@ e_book_backend_groupwise_dispose (GObject *object)
 			g_free (bgw->priv->book_name);
 			bgw->priv->book_name = NULL;
 		}
-		if (bgw->priv->categories_by_id) {
-			g_hash_table_destroy (bgw->priv->categories_by_id);
-			bgw->priv->categories_by_id = NULL;
-		}
-		if (bgw->priv->categories_by_name) {
-			g_hash_table_destroy (bgw->priv->categories_by_name);
-			bgw->priv->categories_by_name = NULL;
-		}
-	
 		if (bgw->priv->cache) {
 			g_object_unref (bgw->priv->cache);
 		}
@@ -2658,8 +2665,6 @@ e_book_backend_groupwise_init (EBookBackendGroupwise *backend)
 	EBookBackendGroupwisePrivate *priv;
                                                                                                                              
 	priv= g_new0 (EBookBackendGroupwisePrivate, 1);
-	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->is_writable = TRUE;
 	priv->is_cache_ready = FALSE;
 	priv->marked_for_offline = FALSE;
