@@ -1,6 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-#include <libgnome/gnome-init.h>
 #include <bonobo/bonobo-main.h>
 #include <stdlib.h>
 #include <libebook/e-book.h>
@@ -13,13 +12,11 @@ main (int argc, char **argv)
 	EBookQuery *query;
 	GList *c, *contacts;
 
-	gnome_program_init("test-ebook", "0.0", LIBGNOME_MODULE, argc, argv, NULL);
-
 	if (bonobo_init (&argc, argv) == FALSE)
 		g_error ("Could not initialize Bonobo");
 
 	if (argc < 3) {
-		printf ("usage: test-search <uid> <query>\n");
+		printf ("usage: test-search <addressbook uri> <query>\n");
 		exit (0);
 	}
 
@@ -29,7 +26,7 @@ main (int argc, char **argv)
 		exit (0);
 	}
 
-	book = e_book_new_from_uri (argv[1], NULL);
+	book = e_book_new_system_addressbook (NULL);
 	if (!book) {
 		printf ("failed to create ebook\n");
 		exit(0);
