@@ -1,3 +1,4 @@
+/*-*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar listener
  *
  * Copyright (C) 2001 Ximian, Inc.
@@ -60,7 +61,7 @@ enum {
 	GET_CHANGES,
 	GET_FREE_BUSY,
 	QUERY,
-	CATEGORIES_CHANGED,
+	AUTH_REQUIRED,
 	BACKEND_ERROR,
 	LAST_SIGNAL
 };
@@ -91,6 +92,9 @@ convert_status (const GNOME_Evolution_Calendar_CallStatus status)
 		return E_CALENDAR_STATUS_AUTHENTICATION_REQUIRED;
 	case GNOME_Evolution_Calendar_UnknownUser:
 		return E_CALENDAR_STATUS_UNKNOWN_USER;
+	case GNOME_Evolution_Calendar_InvalidServerVersion:
+		return E_CALENDAR_STATUS_INVALID_SERVER_VERSION;
+	
 	case GNOME_Evolution_Calendar_OtherError:
 	default:
 		return E_CALENDAR_STATUS_OTHER_ERROR;
@@ -626,6 +630,21 @@ impl_notifyCalSetMode (PortableServer_Servant servant,
 	(* priv->cal_set_mode_fn) (listener, status, mode, priv->fn_data);
 }
 
+static void
+impl_notifyAuthRequired (PortableServer_Servant servant,
+			 CORBA_Environment *ev)
+{
+	ECalListener *listener;
+	ECalListenerPrivate *priv;
+
+	listener = E_CAL_LISTENER (bonobo_object_from_servant (servant));
+	priv = listener->priv;
+
+	if (!priv->notify)
+		return;
+       
+	g_signal_emit (G_OBJECT (listener), signals[AUTH_REQUIRED], 0);
+}
 
 /* ::notifyErrorOccurred method */
 static void
@@ -645,24 +664,6 @@ impl_notifyErrorOccurred (PortableServer_Servant servant,
 	g_signal_emit (G_OBJECT (listener), signals[BACKEND_ERROR], 0, message);
 }
 
-/* ::notifyCategoriesChanged method */
-static void
-impl_notifyCategoriesChanged (PortableServer_Servant servant,
-			      const GNOME_Evolution_Calendar_StringSeq *categories,
-			      CORBA_Environment *ev)
-{
-	ECalListener *listener;
-	ECalListenerPrivate *priv;
-
-	listener = E_CAL_LISTENER (bonobo_object_from_servant (servant));
-	priv = listener->priv;
-
-	if (!priv->notify)
-		return;
-
-	g_signal_emit (G_OBJECT (listener), signals[CATEGORIES_CHANGED], 0, categories);
-}
-
 
 
 /* Object initialization function for the calendar listener */
@@ -735,8 +736,7 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 	klass->epv.notifyQuery = impl_notifyQuery;
 	klass->epv.notifyCalSetMode = impl_notifyCalSetMode;
 	klass->epv.notifyErrorOccurred = impl_notifyErrorOccurred;
-	klass->epv.notifyCategoriesChanged = impl_notifyCategoriesChanged;
-
+	klass->epv.notifyAuthRequired      = impl_notifyAuthRequired;
 	object_class->finalize = e_cal_listener_finalize;
 
 	signals[READ_ONLY] =
@@ -915,14 +915,14 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
-	signals[CATEGORIES_CHANGED] =
-		g_signal_new ("categories_changed",
-			      G_TYPE_FROM_CLASS (klass),
+	signals [AUTH_REQUIRED] =
+		g_signal_new ("auth_required",
+			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ECalListenerClass, categories_changed),
+			      G_STRUCT_OFFSET (ECalListenerClass, auth_required),
 			      NULL, NULL,
-			      e_cal_marshal_VOID__POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+			      e_cal_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 	signals[BACKEND_ERROR] =
 		g_signal_new ("backend_error",
 			      G_TYPE_FROM_CLASS (klass),
