@@ -42,6 +42,7 @@
 #include "camel-private.h"
 #include "camel-groupwise-private.h"
 #include "camel-groupwise-journal.h"
+#include "camel-groupwise-utils.h"
 #include "camel-stream-mem.h"
 #include <e-gw-connection.h>
 #include <e-gw-item.h>
@@ -85,8 +86,7 @@ groupwise_folder_get_message( CamelFolder *folder,
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE(folder->parent_store) ;
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	CamelGroupwiseMessageInfo *mi = NULL ;
-	char *temp_name, *folder_name, *container_id, *body ;
-	CamelInternetAddress *from_addr, *to_addr, *cc_addr, *bcc_addr ;
+	char *temp_name, *folder_name, *container_id, *body , *temp_str = NULL ;
 	GSList *recipient_list, *attach_list ;
 	EGwItemOrganizer *org ;
 	EGwItemType type ;
@@ -99,6 +99,9 @@ groupwise_folder_get_message( CamelFolder *folder,
 	int errno;
 	GList *read_items = NULL ;
 	guint32 item_status ;
+	struct _camel_header_address *ha;
+	char *subs_email;
+	struct _camel_header_address *to_list = NULL, *cc_list = NULL, *bcc_list=NULL;
 	/* see if it is there in cache */
 	CAMEL_SERVICE_LOCK (folder->parent_store, connect_lock);
 
@@ -190,12 +193,6 @@ groupwise_folder_get_message( CamelFolder *folder,
 	org = e_gw_item_get_organizer (item) ;
 	recipient_list = e_gw_item_get_recipient_list (item) ;
 
-	/*Addresses*/
-	from_addr = camel_internet_address_new () ;
-	to_addr = camel_internet_address_new () ;
-	cc_addr = camel_internet_address_new () ;
-	bcc_addr = camel_internet_address_new () ;
-
 	if (recipient_list) {
 		GSList *rl ;
 		char *status_opt = NULL;
@@ -205,21 +202,27 @@ groupwise_folder_get_message( CamelFolder *folder,
 			EGwItemRecipient *recp = (EGwItemRecipient *) rl->data;
 			enabled = recp->status_enabled ;
 
+			if (!recp->email) {
+				ha=camel_header_address_new_group(recp->display_name);
+			} else {
+				ha=camel_header_address_new_name(recp->display_name,recp->email);
+			}
+			
 			if (recp->type == E_GW_ITEM_RECIPIENT_TO) {
 				if (recp->status_enabled) 
 					status_opt = g_strconcat (status_opt ? status_opt : "" , "TO", ";",NULL) ;
-				camel_internet_address_add (to_addr, recp->display_name, recp->email ) ;
-				
+				camel_header_address_list_append(&to_list, ha);
 			} else if (recp->type == E_GW_ITEM_RECIPIENT_CC) {
 				if (recp->status_enabled) 
 					status_opt = g_strconcat (status_opt ? status_opt : "", "CC", ";",NULL) ;
-				camel_internet_address_add (cc_addr, recp->display_name, recp->email ) ;
+				camel_header_address_list_append(&cc_list,ha);
 				
 			} else if (recp->type == E_GW_ITEM_RECIPIENT_BC) {
 				if (recp->status_enabled) 
 					status_opt = g_strconcat (status_opt ? status_opt : "", "BCC", ";",NULL) ;
-				camel_internet_address_add (bcc_addr, recp->display_name, recp->email ) ;
-
+				camel_header_address_list_append(&bcc_list,ha);
+			} else {
+				camel_header_address_unref(ha);
 			}
 			if (recp->status_enabled) {
 				status_opt = g_strconcat (status_opt, 
@@ -240,14 +243,40 @@ groupwise_folder_get_message( CamelFolder *folder,
 			camel_medium_add_header ( CAMEL_MEDIUM (msg), "X-gw-status-opt", (const char *)status_opt) ;
 			g_free (status_opt) ;
 		}
-		
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_TO, to_addr) ;
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_CC, cc_addr) ;
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_BCC, bcc_addr) ;
 	}
-	if (org)
-		camel_internet_address_add (from_addr,org->display_name,org->email) ;
-	
+
+	if(to_list) { 
+		subs_email=camel_header_address_list_encode(to_list);
+		camel_medium_set_header( CAMEL_MEDIUM(msg), "To", subs_email);
+		g_free(subs_email);
+		camel_header_address_list_clear(&to_list);
+	}
+
+	if(cc_list) { 
+		subs_email=camel_header_address_list_encode(cc_list);
+		camel_medium_set_header( CAMEL_MEDIUM(msg), "Cc", subs_email);
+		g_free(subs_email);
+		camel_header_address_list_clear(&cc_list);
+	}
+
+	if(bcc_list) { 
+		subs_email=camel_header_address_list_encode(bcc_list);
+		camel_medium_set_header( CAMEL_MEDIUM(msg), "Bcc", subs_email);
+		g_free(subs_email);
+		camel_header_address_list_clear(&bcc_list);
+	}
+
+	if (org) {
+		if (org->display_name && org->email) {
+			ha=camel_header_address_new_name(org->display_name,org->email);
+		} else {
+			ha=camel_header_address_new_group(org->display_name);
+		}
+		subs_email=camel_header_address_list_encode(ha);	
+		camel_medium_set_header( CAMEL_MEDIUM(msg), "From", subs_email);
+		camel_header_address_unref(ha);
+		g_free(subs_email);
+	}
 	if (e_gw_item_get_reply_request (item)) {
 		char *reply_within; 
 		const char *mess = e_gw_item_get_message (item);
@@ -308,11 +337,16 @@ groupwise_folder_get_message( CamelFolder *folder,
 		camel_object_unref (part) ;
 	}
 	
-	camel_mime_message_set_subject (msg, e_gw_item_get_subject(item) ) ;
+	temp_str = (char *)e_gw_item_get_subject(item) ;
+	if(temp_str) 
+		camel_mime_message_set_subject (msg, temp_str) ;
 	dtstring = e_gw_item_get_creation_date (item) ;
-	if (dtstring) 
-		camel_mime_message_set_date (msg, e_gw_connection_get_date_from_string (dtstring), 0 ) ;
-	camel_mime_message_set_from (msg, from_addr) ;
+	if(dtstring) {
+		int offset = 0 ;
+		time_t time = e_gw_connection_get_date_from_string (dtstring) ;
+		time_t actual_time = camel_header_decode_date (ctime(&time), &offset) ;
+		camel_mime_message_set_date (msg, actual_time, offset) ;
+	}
 	
 
 	/* Attachments
@@ -328,10 +362,12 @@ groupwise_folder_get_message( CamelFolder *folder,
 			int len ;
 			CamelMimePart *part ;
 
-			status = e_gw_connection_get_attachment (cnc, g_strdup(attach->id), 0, -1, (const char **)&attachment, &len) ;
+			status = e_gw_connection_get_attachment (cnc, 
+					                         g_strdup(attach->id), 0, -1, 
+								 (const char **)&attachment, &len) ;
 			if (status != E_GW_CONNECTION_STATUS_OK) {
 				//camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Could not get attachment"));
-				g_message ("Could not get attachment\n") ;
+				g_warning ("Could not get attachment\n") ;
 				continue ;
 			}
 			if (attach && (len !=0) ) {
@@ -368,14 +404,13 @@ groupwise_folder_get_message( CamelFolder *folder,
 		g_free (body) ;
 	/* add to cache */
 	CAMEL_GROUPWISE_FOLDER_LOCK (folder, cache_lock);
-	if (!camel_medium_get_header ( CAMEL_MEDIUM (msg), "X-gw-status-opt") ) {
-		if ((cache_stream = camel_data_cache_add (gw_folder->cache, "cache", uid, NULL))) {
+	if ((cache_stream = camel_data_cache_add (gw_folder->cache, "cache", uid, NULL))) {
 			if (camel_data_wrapper_write_to_stream ((CamelDataWrapper *) msg, cache_stream) == -1
 					|| camel_stream_flush (cache_stream) == -1)
 				camel_data_cache_remove (gw_folder->cache, "cache", uid, NULL);
 			camel_object_unref (cache_stream);
 		}
-	}
+	
 	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
 	CAMEL_SERVICE_UNLOCK (folder->parent_store, connect_lock);
 
@@ -469,17 +504,21 @@ static void
 groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 {
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (folder->parent_store) ;
-	//CamelGroupwiseStorePrivate *priv = gw_store->priv ;
+	CamelGroupwiseStorePrivate *priv = gw_store->priv ;
 	CamelMessageInfo *info ;
 	CamelGroupwiseMessageInfo *gw_info ;
 	GList *items = NULL ;
 	flags_diff_t diff ;
-	//EGwConnection *cnc = cnc_lookup (priv) ;
+	const char *container_id;
+	EGwConnectionStatus status ;
+	EGwConnection *cnc = cnc_lookup (priv) ;
 	int count, i ;
-	g_print ("groupwise sync\n") ;
+
 	if (((CamelOfflineStore *) gw_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) 
 		return ;
 	
+	container_id =  camel_groupwise_store_container_id_lookup (gw_store, folder->name) ;
+	
 	CAMEL_SERVICE_LOCK (folder->parent_store, connect_lock);
 
 	count = camel_folder_summary_count (folder->summary) ;
@@ -487,7 +526,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		info = camel_folder_summary_index (folder->summary, i) ;
 		gw_info = (CamelGroupwiseMessageInfo *) info ;
 
-		if (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED) {
+		if (gw_info && (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
 			do_flags_diff (&diff, gw_info->server_flags, gw_info->info.flags) ;
 			diff.changed &= folder->permanent_flags;
 
@@ -497,6 +536,12 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			else {
 				if (diff.changed & CAMEL_MESSAGE_SEEN)
 					items = g_list_append (items, (char *)camel_message_info_uid(info));
+				if (diff.changed & CAMEL_MESSAGE_DELETED) {
+					const char *uid = camel_message_info_uid (info) ;
+					status = e_gw_connection_remove_item (cnc, container_id, uid);
+					if (status == E_GW_CONNECTION_STATUS_OK) 
+						camel_folder_summary_remove_uid (folder->summary,uid) ;
+				}
 			}
 		}
 		
@@ -504,6 +549,9 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 /*	if (items) 
 		e_gw_connection_mark_read (cnc, items) ;*/
+
+	if (expunge)
+		e_gw_connection_purge_deleted_items (cnc) ;
 	
 	camel_folder_summary_save (folder->summary);
 
@@ -583,11 +631,7 @@ groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 	GList *list = NULL;
 	GSList *slist = NULL, *sl ;
 	char *container_id = NULL ;
-	char *cache_file_name ;
-	static time_t mod_time = 0 ;
-	char time_string[100] = {0} ;
-	const struct tm *tm ;
-	struct stat buf ;
+	char *time_string = NULL, *t_str = NULL ;
 
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->name)) ;
 	if (!container_id) {
@@ -604,23 +648,19 @@ groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 		gw_folder->need_refresh = TRUE ;
 	}
 
-	if (!mod_time) {
-		cache_file_name = g_strdup (folder->summary->summary_path) ;
-		stat (cache_file_name, &buf) ;
-		mod_time = buf.st_mtime ;
-		g_free (cache_file_name) ;
-	}
-
-	tm = gmtime (&mod_time) ;
-	strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm) ;
+	time_string =  ((CamelGroupwiseSummary *) folder->summary)->time_string;
+	t_str = g_strdup (time_string);
 	
 	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
-
+	/* FIXME send the time stamp which the server sends */
 	status = e_gw_connection_get_quick_messages (cnc, container_id,
-					"default distribution created attachments subject",
-					time_string, "New", "Mail", NULL, -1, &slist) ;
+					"distribution created attachments subject",
+					&t_str, "New", "Mail", NULL, -1, &slist) ;
+	
+	g_free (t_str), t_str = NULL;
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 		g_free (container_id) ;
 		return ;
 	}
@@ -628,13 +668,33 @@ groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 	for ( sl = slist ; sl != NULL; sl = sl->next) {
 		list = g_list_append (list, sl->data) ;
 	}
+	g_slist_free (slist);
+	slist = NULL;
+	t_str = g_strdup (time_string);
+	/* FIXME send the time stamp which the server sends */
+	status = e_gw_connection_get_quick_messages (cnc, container_id,
+				"distribution created attachments subject",
+				&t_str, "Modified", "Mail", NULL, -1, &slist) ;
+	g_free (t_str), t_str = NULL;
+	if (status != E_GW_CONNECTION_STATUS_OK) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		g_free (container_id) ;
+		return ;
+	}
+	for ( sl = slist ; sl != NULL; sl = sl->next) {
+		list = g_list_append (list, sl->data) ;
+	}
+
+	g_slist_free (slist);
+	slist = NULL;
 
 	if (gw_store->current_folder != folder) {
 		gw_store->current_folder = folder ;
 	} else if(gw_folder->need_rescan) {
-		gw_update_summary (folder, list, ex) ;
 	}
-
+	
+	gw_update_summary (folder, list, ex) ;
 
 	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 
@@ -652,7 +712,7 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	GSList *attach_list = NULL ;
 	guint32 item_status, status_flags;
 	CamelFolderChangeInfo *changes = NULL ;
-	int scount, got = g_list_length (item_list) ;
+	int scount ;
 	
 //	CAMEL_SERVICE_ASSERT_LOCKED (gw_store, connect_lock);
 
@@ -674,10 +734,8 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		if (mi) {
 			/*Message exists*/
 			item_status = e_gw_item_get_item_status (item);
-			if (item_status & E_GW_ITEM_STAT_READ)
-				status_flags |= CAMEL_MESSAGE_SEEN;
-			if (item_status & E_GW_ITEM_STAT_DELETED)
-				status_flags |= CAMEL_MESSAGE_DELETED;
+			/*if (item_status & E_GW_ITEM_STAT_DELETED)
+				status_flags |= CAMEL_MESSAGE_DELETED;*/
 			if (item_status & E_GW_ITEM_STAT_REPLIED)
 				status_flags |= CAMEL_MESSAGE_ANSWERED;
 			mi->info.flags |= status_flags;
@@ -703,8 +761,8 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 			item_status = e_gw_item_get_item_status (item);
 			if (item_status & E_GW_ITEM_STAT_READ)
 				status_flags |= CAMEL_MESSAGE_SEEN;
-			if (item_status & E_GW_ITEM_STAT_DELETED)
-				status_flags |= CAMEL_MESSAGE_DELETED;
+			/*if (item_status & E_GW_ITEM_STAT_DELETED)
+				status_flags |= CAMEL_MESSAGE_DELETED;*/
 			if (item_status & E_GW_ITEM_STAT_REPLIED)
 				status_flags |= CAMEL_MESSAGE_ANSWERED;
 			mi->info.flags |= status_flags;
@@ -721,8 +779,9 @@ gw_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex)
 
 			temp_date = e_gw_item_get_creation_date(item) ;
 			if (temp_date) {
-				date = e_gw_connection_format_date_string(temp_date) ;
-				mi->info.date_sent = mi->info.date_received = e_gw_connection_get_date_from_string (date) ;
+				time_t time = e_gw_connection_get_date_from_string (temp_date) ;
+				time_t actual_time = camel_header_decode_date (ctime(&time), NULL) ;
+				mi->info.date_sent = mi->info.date_received = actual_time ;
 			}
 			mi->info.uid = g_strdup (e_gw_item_get_id (item)) ;
 			mi->info.subject = g_strdup (e_gw_item_get_subject(item)) ;
@@ -926,8 +985,8 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 	EGwConnection *cnc;
 	EGwConnectionStatus status ;
 	CamelFolderChangeInfo *changes ;
-	GList *l, *n, *item_ids = NULL;
 	int i, max;
+	gboolean delete = FALSE ;
 	
 	CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
 	
@@ -935,30 +994,26 @@ groupwise_expunge (CamelFolder *folder, CamelException *ex)
 
 	cnc = cnc_lookup (priv) ;
 	
+	container_id =  g_strdup (camel_groupwise_store_container_id_lookup (groupwise_store, folder->name)) ;
+
 	max = camel_folder_summary_count (folder->summary);
 	for (i = 0; i < max; i++) {
 		info = camel_folder_summary_index (folder->summary, i);
 		ginfo = (CamelGroupwiseMessageInfo *) info;
-		if (ginfo->info.flags & CAMEL_MESSAGE_DELETED) {
-			item_ids = g_list_append (item_ids, g_strdup (camel_message_info_uid (info)));
-			camel_folder_change_info_remove_uid (changes, (char *) item_ids->data);
+		if (ginfo && (ginfo->info.flags & CAMEL_MESSAGE_DELETED)) {
+			const char *uid = camel_message_info_uid (info) ;
+			status = e_gw_connection_remove_item (cnc, container_id, uid);
+			if (status == E_GW_CONNECTION_STATUS_OK) {
+				camel_folder_change_info_remove_uid (changes, (char *) uid);
+				camel_folder_summary_remove_uid (folder->summary, uid) ;
+				delete = TRUE ;
+			}
 		}
 		camel_message_info_free (info);
 	}
-	
-	container_id =  g_strdup (camel_groupwise_store_container_id_lookup (groupwise_store, folder->name)) ;
-	status = e_gw_connection_remove_items (cnc, container_id, item_ids);
-	if (status == E_GW_CONNECTION_STATUS_OK)
-		camel_object_trigger_event (CAMEL_OBJECT (folder), "folder_changed", changes);
-	else
-		g_print ("ERROR!!! Could not delete items\n") ;
-	
-	l = item_ids;
-	while (l != NULL) {
-		n = l->next;
-		g_list_free_1 (l);
-		l = n;
-	}
+
+	if (delete)
+		camel_object_trigger_event (CAMEL_OBJECT (folder), "folder_changed", changes) ;
 	
 	CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
 	
@@ -1106,6 +1161,7 @@ convert_to_calendar (EGwItem *item, char **str, int *len)
 	*str = g_strconcat ("BEGIN:VCALENDAR","\n", 
 			    "METHOD:REQUEST", "\n", 
 			    "BEGIN:VEVENT", "\n", 
+			    "UID:", e_gw_item_get_icalid (item), "\n",
 			    "X-GWITEM-TYPE:APPOINTMENT", "\n", 
 			    "DTSTART:", e_gw_item_get_start_date (item), "\n",
 			    "SUMMARY:", e_gw_item_get_subject (item), "\n",
