@@ -34,7 +34,9 @@
 #include <string.h>
 #include <errno.h>
 #include <glib.h>
-
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "camel-groupwise-store.h"
 #include "camel-groupwise-summary.h"
@@ -49,7 +51,8 @@
 #include "camel-folder.h" 
 #include "camel-private.h"
 
-#define d(x) printf(x);
+#define d(x) 
+#define CURSOR_ITEM_LIMIT 300
 
 struct _CamelGroupwiseStorePrivate {
 	char *server_name;
@@ -182,8 +185,6 @@ camel_groupwise_store_finalize (CamelObject *object)
 			g_object_unref (priv->cnc);
 			priv->cnc = NULL;
 		}
-		g_free (groupwise_store->priv);
-		groupwise_store->priv = NULL;
 
 		if (priv->storage_path)
 			g_free(priv->storage_path) ;
@@ -203,6 +204,8 @@ camel_groupwise_store_finalize (CamelObject *object)
 			g_hash_table_foreach (priv->parent_hash, free_hash, NULL) ;
 			g_hash_table_destroy (priv->parent_hash) ;
 		}
+		g_free (groupwise_store->priv);
+		groupwise_store->priv = NULL;
 	}
 
 }
@@ -294,9 +297,9 @@ groupwise_store_construct (CamelService *service, CamelSession *session,
 	/*soap port*/
 	property_value =  camel_url_get_param (url, "soap_port");
 	if (property_value == NULL)
-		priv->port = g_strdup ("7181");
+		priv->port = g_strdup ("7191");
 	else if(strlen(property_value) == 0)
-		priv->port = g_strdup ("7181");
+		priv->port = g_strdup ("7191");
 	else
 		priv->port = g_strdup (property_value);
 
@@ -306,7 +309,7 @@ groupwise_store_construct (CamelService *service, CamelSession *session,
 	priv->parent_hash = g_hash_table_new (g_str_hash, g_str_equal) ;
 
 	/*ssl*/
-	priv->use_ssl = g_strdup (camel_url_get_param (url, "soap_ssl"));
+	priv->use_ssl = g_strdup (camel_url_get_param (url, "use_ssl"));
 	
 	store->flags = 0; //XXX: Shouldnt do this....
 	
@@ -335,20 +338,19 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
 	char *errbuf = NULL;
 	gboolean authenticated = FALSE;
-	const char *auth_domain;
 	char *uri;
 
 	CAMEL_SERVICE_ASSERT_LOCKED (groupwise_store, connect_lock);
-	auth_domain = camel_url_get_param (service->url, "auth-domain");
-	if (priv->use_ssl) 
+	if (priv->use_ssl && !g_str_equal (priv->use_ssl, "never")) 
 		uri = g_strconcat ("https://", priv->server_name, ":", priv->port, "/soap", NULL);
 	else 
 		uri = g_strconcat ("http://", priv->server_name, ":", priv->port, "/soap", NULL);
 	service->url->passwd = NULL;
+
 	while (!authenticated) {
 		if (errbuf) {
 			/* We need to un-cache the password before prompting again */
-			camel_session_forget_password (session, service, auth_domain, "password", ex);
+			camel_session_forget_password (session, service, "Groupwise", "password", ex);
 			g_free (service->url->passwd);
 			service->url->passwd = NULL;
 		}
@@ -362,7 +364,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 						  service->url->user,
 						  service->url->host);
 			service->url->passwd =
-				camel_session_get_password (session, service, auth_domain,
+				camel_session_get_password (session, service, "Groupwise",
 							    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
 			g_free (prompt);
 			g_free (errbuf);
@@ -378,7 +380,12 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 		
 				
 		priv->cnc = e_gw_connection_new (uri, priv->user, service->url->passwd);
-		if (!priv->cnc) {
+		if (!E_IS_GW_CONNECTION(priv->cnc) && priv->use_ssl && g_str_equal (priv->use_ssl, "when-possible")) {
+			char *http_uri = g_strconcat ("http://", uri + 8, NULL);
+			priv->cnc = e_gw_connection_new (http_uri, priv->user, service->url->passwd);
+			g_free (http_uri);
+		}
+		if (!E_IS_GW_CONNECTION(priv->cnc)) {
 			errbuf = g_strdup_printf (_("Unable to authenticate "
 					    "to GroupWise server."));
 						  
@@ -397,7 +404,8 @@ groupwise_connect (CamelService *service, CamelException *ex)
 	CamelGroupwiseStore *store = CAMEL_GROUPWISE_STORE (service);
 	CamelGroupwiseStorePrivate *priv = store->priv;
 	CamelGroupwiseStoreNamespace *ns;
-	
+	CamelSession *session = service->session;
+
 	d("in groupwise store connect\n");
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
@@ -409,19 +417,27 @@ groupwise_connect (CamelService *service, CamelException *ex)
 		CAMEL_SERVICE_UNLOCK (service, connect_lock);
 		return TRUE;
 	}
-	
+
 	if (!groupwise_auth_loop (service, ex)) {
 		CAMEL_SERVICE_UNLOCK (service, connect_lock);
 		camel_service_disconnect (service, TRUE, NULL);
 		return FALSE;
 	}
-	
+
+	if (!e_gw_connection_get_version (priv->cnc)) {
+		camel_session_alert_user(session, 
+				CAMEL_SESSION_ALERT_WARNING, 
+				_("Some features may not work correctly with your current server version"),
+				FALSE);
+
+	}
+
 	camel_store_summary_save ((CamelStoreSummary *) store->summary);
-	
+
 	ns = camel_groupwise_store_summary_namespace_new (store->summary, priv->storage_path, '/');
 	camel_groupwise_store_summary_namespace_set (store->summary, ns);
 	camel_store_summary_save ((CamelStoreSummary *) store->summary);
-	
+
 	CAMEL_SERVICE_UNLOCK (service, connect_lock);
 	if (E_IS_GW_CONNECTION (priv->cnc))
 		return TRUE;
@@ -435,10 +451,12 @@ groupwise_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	
+	CAMEL_SERVICE_LOCK (groupwise_store, connect_lock) ;
 	if (groupwise_store->priv->cnc) {
 		g_object_unref (groupwise_store->priv->cnc);
 		groupwise_store->priv->cnc = NULL;
 	}
+	CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock) ;
 	
 	return TRUE;
 }
@@ -453,36 +471,44 @@ groupwise_store_query_auth_types (CamelService *service, CamelException *ex)
 	return auth_types;
 }
 
+static GMutex *mutex = NULL ;
 
-/*****************/
 static CamelFolder * 
 groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags, CamelException *ex)
 {
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE (store) ;
 	CamelGroupwiseStorePrivate *priv = gw_store->priv ;
 	CamelFolder *folder ;
+	CamelGroupwiseSummary *summary;
+	char *time_string = NULL;
 	char *storage_path, *folder_dir, *temp_str,*container_id ;
 	const char *temp_name;
 	EGwConnectionStatus status ;
 	GList *list = NULL ;
-	
-	g_print ("||GW:Get folder\n") ;
+	GSList *slist = NULL, *sl ;
+	gboolean done = FALSE ;
+	const char *position = E_GW_CURSOR_POSITION_END; 
+	int count = 0, cursor, summary_count = 0 ;
 	
 	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
         folder_dir = e_path_to_physical (storage_path, folder_name);
 	g_free (storage_path);
 	
+	CAMEL_SERVICE_LOCK (gw_store, connect_lock) ;
+	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		if (!folder_dir || access (folder_dir, F_OK) != 0) {
 			g_free (folder_dir);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
 					      _("No such folder %s"), folder_name);
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock) ;
 			return NULL;
 		}
 		
 		folder = camel_gw_folder_new (store, folder_name, folder_dir, ex);
 		g_free (folder_dir);
-		
+			
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock) ;
 		return folder;
 	}
 	
@@ -496,33 +522,154 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		container_id = 	g_strdup (g_hash_table_lookup (priv->name_hash, g_strdup(temp_str))) ;
 	}
 
-	camel_operation_start (NULL, _("Fetching summary information for new messages"));
-
-	status = e_gw_connection_get_items (priv->cnc, container_id, "default attachments distribution created subject message notification", NULL, &list) ;
-	if (status != E_GW_CONNECTION_STATUS_OK) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+	folder = camel_gw_folder_new (store, folder_name, folder_dir, ex) ;
+	if (!folder) {
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock) ;
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed")) ;
+		g_free (folder_dir) ;
 		g_free (container_id) ;
-		camel_operation_end (NULL);
-		g_free (folder_dir);
-		return NULL;
+		return NULL ;
 	}
-	
-	folder = camel_gw_folder_new (store,folder_name, folder_dir, ex);
-	camel_operation_end (NULL);
-	g_free (folder_dir);
-	
-	if(folder) {
-		CamelException local_ex ;
 
-		gw_store->current_folder = folder ;
-		camel_object_ref (folder) ;
-		camel_exception_init (&local_ex) ;
+	if (!mutex)
+		mutex = g_mutex_new () ;
+	summary = (CamelGroupwiseSummary *) folder->summary;
+	if (summary->time_string && *(summary->time_string))
+		time_string = summary->time_string;	
+	else {
+		time_t mod_time = time (0) ;
+		const struct tm *tm;
+		/*TODO get current time here */
+		tm = gmtime (&mod_time);
+		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	}
+
+	summary_count = camel_folder_summary_count (folder->summary) ;
+	if(summary_count) {
+		char  *t_str = NULL;
 		
-		gw_update_summary (folder, list,  ex) ;
+		t_str = g_strdup (time_string);
+		/* FIXME send the time stamp which the server sends */
+		status = e_gw_connection_get_quick_messages (priv->cnc, container_id,
+				"distribution attachments subject created",
+				&t_str, "New", "Mail", NULL, -1, &slist) ;
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			//camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock) ;
+			camel_operation_end (NULL);
+			g_free (container_id) ;
+			gw_store->current_folder = folder ;
+			camel_object_ref (folder) ;
+			camel_folder_summary_save(folder->summary) ;
+			return folder ;
+			//return NULL ;
+		}
 
-		camel_folder_summary_save(folder->summary) ;
+		/* store t_str into the summary */	
+		if (summary->time_string)
+			g_free (summary->time_string);
+		summary->time_string = g_strdup (t_str);
+		g_free (t_str), t_str = NULL;
+		for ( sl = slist ; sl != NULL; sl = sl->next) {
+			list = g_list_append (list, sl->data) ;
+		}
+
+		g_slist_free (slist);
+		slist = NULL;
+		t_str = g_strdup (time_string);
+		/* FIXME send the time stamp which the server sends */
+		status = e_gw_connection_get_quick_messages (priv->cnc, container_id,
+					"distribution attachments subject created",
+					&t_str, "Modified", "Mail", NULL, -1, &slist) ; 
+		g_free (t_str), t_str = NULL;
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			//camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock) ;
+			camel_operation_end (NULL);
+			g_free (container_id) ;
+			gw_store->current_folder = folder ;
+			camel_object_ref (folder) ;
+			camel_folder_summary_save(folder->summary) ;
+			return folder ;
+			//return NULL ;
+		}
+
+		for ( sl = slist ; sl != NULL; sl = sl->next) {
+			list = g_list_append (list, sl->data) ;
+		}
+		g_slist_free (slist);
+		slist = NULL;
+		gw_update_summary (folder, list,  ex) ;
+		camel_operation_end (NULL);
+
+	} else {
+		g_print ("\n\nNo summary as yet : using get cursor request\n\n") ;
+		g_mutex_lock (mutex) ;
+
+		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
+				"attachments distribution created subject",
+				NULL,
+				&cursor) ;
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+			g_mutex_unlock (mutex) ;
+			return NULL ;
+		}
+
+		camel_operation_start (NULL, _("Fetching summary information for new messages"));
+		camel_folder_summary_clear (folder->summary) ;
+
+		while (!done) {
+			int temp = 0 ;
+			status = e_gw_connection_read_cursor (priv->cnc, container_id, 
+							      cursor, FALSE, 
+							      CURSOR_ITEM_LIMIT, position, &list) ;
+			if (status != E_GW_CONNECTION_STATUS_OK) {
+				CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+				camel_operation_end (NULL);
+				g_mutex_unlock (mutex) ;
+				g_free (container_id) ;
+				g_free (folder_dir);
+				return NULL ;
+			}
+
+			temp = g_list_length (list) ;
+			count += temp ;
+			
+			if (summary->time_string)
+				g_free (summary->time_string);
+			summary->time_string = g_strdup (e_gw_connection_get_server_time (priv->cnc));
+			gw_update_summary (folder, list,  ex) ;
+			if (temp == count)
+				camel_operation_progress (NULL, 0) ;
+			if (temp < count)
+				camel_operation_progress (NULL, 100 - (temp * 100/count)) ;
+			if (temp >count)
+				camel_operation_progress (NULL, 100 - (count * 100/temp)) ;
+			if (!list  || temp == 0)
+				done = TRUE;
+			g_list_free (list);
+			list = NULL;
+			position = E_GW_CURSOR_POSITION_CURRENT;
+		}
+
+		e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor) ;
+
+		camel_operation_end (NULL);
+		g_mutex_unlock (mutex) ;
 	}
+
+	gw_store->current_folder = folder ;
+	camel_object_ref (folder) ;
 	
+	camel_folder_summary_save(folder->summary) ;
+		
+	g_free (container_id) ;
+	g_free (folder_dir);
+
+	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+
 	return folder;
 }
 
@@ -628,17 +775,21 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	int status;
 	GPtrArray *folders;
 	GList *folder_list = NULL, *temp_list = NULL ;
-	const char *url, *top_folder;
+	const char *url, *top_folder, *temp_url ;
 	char *temp_str = NULL, *folder_real = NULL ;
 	CamelFolderInfo *info = NULL ;
-	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
+
+	CAMEL_SERVICE_LOCK (store, connect_lock) ;
+
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
 		return groupwise_get_folder_info_offline (store, top, flags, ex);
+	}
 	
-	g_print ("||GW:Get folder info online\n") ;
 	if (!E_IS_GW_CONNECTION( priv->cnc)) {
 		if (!groupwise_connect (CAMEL_SERVICE(store), ex)) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE, _("Authentication failed"));
+			CAMEL_SERVICE_UNLOCK (store, connect_lock);
 			return NULL;
 		}
 	}
@@ -651,22 +802,21 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 			temp_str++ ;
 			top_folder = g_hash_table_lookup (priv->name_hash, temp_str) ;	
 		} else
-			top_folder = g_hash_table_lookup (priv->name_hash, top) ;	
-
+			top_folder = g_hash_table_lookup (priv->name_hash, top) ;
+		/* 'top' is a valid path, but doesnt have a container id
+		 *  return NULL */
+		if (!top_folder)
+			return NULL ;
 	}
 
-	
 	status = e_gw_connection_get_container_list (priv->cnc, top_folder, &folder_list);
 	if (status != E_GW_CONNECTION_STATUS_OK ) {
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
 		/*FIX ME set the camel exception id*/
 		return NULL;
 	}
-	status = e_gw_connection_get_container_list (priv->cnc, top_folder, &temp_list);
-	if (status != E_GW_CONNECTION_STATUS_OK ) {
-		/*FIX ME set the camel exception id*/
-		return NULL;
-	}
-	//	*temp_list = &folder_list ;
+	
+	temp_list = folder_list ;
 	
 	folders = g_ptr_array_new();
 	
@@ -675,6 +825,11 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 				    CAMEL_URL_HIDE_PARAMS|
 				    CAMEL_URL_HIDE_AUTH) );
 
+	if ( url[strlen(url) - 1] != '/') {
+		temp_url = g_strconcat (url, "/", NULL) ;
+		g_free ((char *)url) ;
+		url = temp_url ;
+	}
 	/*Populate the hash table for finding the mapping from container id <-> folder name*/
 	for (;temp_list != NULL ; temp_list = g_list_next (temp_list) ) {
 		const char *name, *id, *parent ;
@@ -697,31 +852,49 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	}
 
 	
-
 	for (; folder_list != NULL; folder_list = g_list_next(folder_list)) {
 		CamelFolderInfo *fi;
 		const char *parent ;
 		gchar *par_name = NULL;
-		
-		if (e_gw_container_is_root (E_GW_CONTAINER(folder_list->data))) 
+		EGwContainer *container = E_GW_CONTAINER (folder_list->data) ;
+		EGwContainerType type = e_gw_container_get_container_type (container) ;
+		const char *name = e_gw_container_get_name (container) ;
+
+		if (e_gw_container_is_root (container)) 
+			continue ;
+		if ( (type == E_GW_CONTAINER_TYPE_CALENDAR) || (type == E_GW_CONTAINER_TYPE_CONTACTS) )
 			continue ;
 
 		fi = g_new0 (CamelFolderInfo, 1);
 
+		if (type == E_GW_CONTAINER_TYPE_INBOX)
+			fi->flags |= CAMEL_FOLDER_TYPE_INBOX;
+/*		if (type == E_GW_CONTAINER_TYPE_TRASH)
+			fi->flags |= CAMEL_FOLDER_TYPE_TRASH ;*/
+
+		if ( (type == E_GW_CONTAINER_TYPE_INBOX) ||
+		     (type == E_GW_CONTAINER_TYPE_OUTBOX) ||
+		     (type == E_GW_CONTAINER_TYPE_DOCUMENTS) ||
+		     (type == E_GW_CONTAINER_TYPE_QUERY) ||
+		     (type == E_GW_CONTAINER_TYPE_CHECKLIST) ||
+		     (type == E_GW_CONTAINER_TYPE_DRAFT) ||
+		     (type == E_GW_CONTAINER_TYPE_CABINET) ||
+		     (type == E_GW_CONTAINER_TYPE_TRASH) )
+			fi->flags |= CAMEL_FOLDER_SYSTEM ;
 		/*
 		  parent_hash contains the "parent id <-> container id" combination. So we form
 		  the path for the full name in camelfolder info by looking up the hash table until
 		  NULL is found
 		*/
 
-		parent = e_gw_container_get_parent_id (E_GW_CONTAINER (folder_list->data)) ;
+		parent = e_gw_container_get_parent_id (container) ;
 		par_name = g_hash_table_lookup (priv->id_hash, parent) ;
 
 		if (par_name != NULL) {
 			gchar *temp_parent = NULL, *temp = NULL ;
-			gchar *str = g_strconcat (par_name,"/",e_gw_container_get_name (E_GW_CONTAINER (folder_list->data)), NULL) ;
+			gchar *str = g_strconcat (par_name,"/",name, NULL) ;
 
-			fi->name = g_strdup (e_gw_container_get_name (E_GW_CONTAINER (folder_list->data)) ) ;
+			fi->name = g_strdup (name) ;
 
 			temp_parent = g_hash_table_lookup (priv->parent_hash, parent) ;
 			while (temp_parent) {
@@ -739,22 +912,37 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 			g_free (str) ;
 		}
 		else {
-			fi->name =  fi->full_name = g_strdup (e_gw_container_get_name (E_GW_CONTAINER (folder_list->data)));
-			fi->uri = g_strconcat (url, "", e_gw_container_get_name(E_GW_CONTAINER(folder_list->data)), NULL) ;
+			fi->name =  fi->full_name = g_strdup (name);
+			fi->uri = g_strconcat (url, "", name, NULL) ;
+		}
 
+		if (e_gw_container_get_is_shared_to_me (container))
+                        fi->flags |= CAMEL_FOLDER_SHARED_TO_ME;
+                                                                                                                             
+                if (e_gw_container_get_is_shared_by_me (container))
+                        fi->flags |= CAMEL_FOLDER_SHARED_BY_ME;
+
+		fi->total = e_gw_container_get_total_count (container) ;
+		fi->unread = e_gw_container_get_unread_count (container) ;
+		/*refresh info*/
+		if (groupwise_store->current_folder &&
+		    strcmp (groupwise_store->current_folder->full_name, fi->full_name) == 0) {
+
+			CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS (groupwise_store->current_folder))->refresh_info(groupwise_store->current_folder, ex);
 		}
-		
 		g_ptr_array_add (folders, fi);
-		fi->total = e_gw_container_get_total_count (E_GW_CONTAINER (folder_list->data)) ;
-		fi->unread = e_gw_container_get_unread_count (E_GW_CONTAINER (folder_list->data)) ;
-		
+		name = NULL ;
 	}
+	g_free ((char *)url) ;
 	if ( (top != NULL) && (folders->len == 0)) {
 		/*temp_str already contains the value if any*/
-		if (temp_str)
+		if (temp_str) {
+			CAMEL_SERVICE_UNLOCK (store, connect_lock);
 			return groupwise_build_folder_info (groupwise_store, NULL, temp_str ) ;
-		else
+		} else {
+			CAMEL_SERVICE_UNLOCK (store, connect_lock);
 			return groupwise_build_folder_info (groupwise_store, NULL, top ) ;
+		}
 	}
 	info = camel_folder_info_build (folders, top, '/', TRUE) ;
 	g_ptr_array_free (folders, TRUE) ;
@@ -762,6 +950,7 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	folder_real = camel_groupwise_store_summary_path_to_full(groupwise_store->summary, top, '/') ;
 	camel_groupwise_store_summary_add_from_full(groupwise_store->summary, folder_real, '/') ;
 	camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary) ;
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 	return info ;
 }
 
@@ -801,6 +990,7 @@ groupwise_create_folder(CamelStore *store,
 			return NULL;
 		}
 	}
+	CAMEL_SERVICE_LOCK (store, connect_lock) ;
 	status = e_gw_connection_create_folder(priv->cnc,parent_id,folder_name, &child_container_id) ;
 	if (status == E_GW_CONNECTION_STATUS_OK) {
 		root = groupwise_build_folder_info(groupwise_store, parent_name,folder_name) ;
@@ -812,6 +1002,7 @@ groupwise_create_folder(CamelStore *store,
 
 		camel_object_trigger_event (CAMEL_OBJECT (store), "folder_created", root);
 	}
+	CAMEL_SERVICE_UNLOCK (store, connect_lock) ;
 	return root ;
 }
 
@@ -838,6 +1029,7 @@ groupwise_delete_folder(CamelStore *store,
 	} else
 		container = g_hash_table_lookup (priv->name_hash, folder_name) ;
 
+	CAMEL_SERVICE_LOCK (store, connect_lock) ;
 	
 	status = e_gw_connection_remove_item (priv->cnc, container, container) ;
 
@@ -853,6 +1045,7 @@ groupwise_delete_folder(CamelStore *store,
 		
 		g_hash_table_remove (priv->parent_hash, container) ;
 	}
+	CAMEL_SERVICE_UNLOCK (store, connect_lock) ;
 }
 						     
 
@@ -868,24 +1061,41 @@ groupwise_rename_folder(CamelStore *store,
 	CamelGroupwiseStorePrivate  *priv = groupwise_store->priv;
 	char *oldpath, *newpath, *storepath ;
 	const char *container_id ;
+	char *temp_old = NULL, *temp_new = NULL ;
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename GroupWise folders in offline mode."));
 		return;
 	}
-	
-	container_id = camel_groupwise_store_container_id_lookup (groupwise_store, old_name) ;
-	if (!container_id || e_gw_connection_rename_folder (priv->cnc, container_id , new_name) != E_GW_CONNECTION_STATUS_OK) {
+	CAMEL_SERVICE_LOCK (store, connect_lock) ;
+	temp_old = strrchr (old_name,'/') ;
+	if (temp_old) {
+		temp_old++ ;
+		container_id = camel_groupwise_store_container_id_lookup (groupwise_store, temp_old) ;
+	} else
+		container_id = camel_groupwise_store_container_id_lookup (groupwise_store, old_name) ;
+
+	temp_new = strrchr (new_name, '/') ;
+	if (temp_new) 
+		temp_new++ ;
+	else
+		temp_new = (char *)new_name ;
+		
+	if (!container_id || e_gw_connection_rename_folder (priv->cnc, container_id , temp_new) != E_GW_CONNECTION_STATUS_OK) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename Groupwise folder `%s' to `%s'"),
 				      old_name, new_name);
+		CAMEL_SERVICE_UNLOCK (store, connect_lock) ;
 		return ;
 	}
 
-	g_hash_table_replace (priv->id_hash, g_strdup(container_id), g_strdup(new_name)) ;
+	g_hash_table_replace (priv->id_hash, g_strdup(container_id), g_strdup(temp_new)) ;
 
-        g_hash_table_insert (priv->name_hash, g_strdup(new_name), g_strdup(container_id)) ;
-	g_hash_table_remove (priv->name_hash, g_strdup(old_name)) ;
+        g_hash_table_insert (priv->name_hash, g_strdup(temp_new), g_strdup(container_id)) ;
 	
+	if (temp_old)
+		g_hash_table_remove (priv->name_hash, g_strdup(temp_old)) ;
+	else
+		g_hash_table_remove (priv->name_hash, g_strdup(old_name)) ;
 	/*FIXME:Update all the id in the parent_hash*/
 
 	storepath = g_strdup_printf ("%s/folders", priv->storage_path) ;
@@ -901,7 +1111,7 @@ groupwise_rename_folder(CamelStore *store,
 
 	g_free (oldpath) ;
 	g_free (newpath) ;
- 
+	CAMEL_SERVICE_UNLOCK (store, connect_lock) ;
 }
 
 char * 
@@ -1064,7 +1274,6 @@ update_folder_counts (CamelGroupwiseStore *gw_store, CamelFolderInfo *fi, CamelE
 					fi->total = camel_folder_get_message_count(gw_store->current_folder);
 				} else {
 					/*Update the counts for all the other folders*/
-					g_print ("|| GW:Other folder:%s||\n", fi->full_name) ;
 					/*TODO: We have to somehow get the folder counts*/
 					folder = camel_object_bag_peek(CAMEL_STORE(gw_store)->folders, fi->full_name);
 					if (folder) {
