@@ -314,10 +314,13 @@ camel_imap_command_response (CamelImapStore *store, char **response,
 		respbuf = imap_read_untagged (store, respbuf, ex);
 		if (!respbuf)
 			type = CAMEL_IMAP_RESPONSE_ERROR;
-		else if (!strncasecmp (respbuf, "* OK [ALERT]", 12)) {
+		else if (!g_ascii_strncasecmp (respbuf, "* OK [ALERT]", 12)
+			 || !g_ascii_strncasecmp (respbuf, "* NO [ALERT]", 12)
+			 || !g_ascii_strncasecmp (respbuf, "* BAD [ALERT]", 13)) {
 			char *msg;
 
 			/* for imap ALERT codes, account user@host */
+			/* we might get a ']' from a BAD response since we +12, but who cares? */
 			msg = g_strdup_printf(_("Alert from IMAP server %s@%s:\n%s"),
 					      ((CamelService *)store)->url->user, ((CamelService *)store)->url->host, respbuf+12);
 			camel_session_alert_user(((CamelService *)store)->session, CAMEL_SESSION_ALERT_WARNING, msg, FALSE);
@@ -577,7 +580,9 @@ camel_imap_response_free (CamelImapStore *store, CamelImapResponse *response)
 			number = strtoul (resp + 2, &p, 10);
 			if (!g_ascii_strcasecmp (p, " EXISTS")) {
 				exists = number;
-			} else if (!strcasecmp (p, " EXPUNGE")) {
+			} else if (!g_ascii_strcasecmp (p, " EXPUNGE")
+				   || !g_ascii_strcasecmp(p, " XGWMOVE")) {
+				/* XGWMOVE response is the same as an EXPUNGE response */
 				if (!expunged) {
 					expunged = g_array_new (FALSE, FALSE,
 								sizeof (int));
@@ -808,7 +813,7 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 				outptr += sprintf (outptr, "%s", string);
 			} else {
 				if (store->capabilities & IMAP_CAPABILITY_LITERALPLUS) {
-					outptr += sprintf (outptr, "{%d+}\r\n%s", strlen (string), string);
+					outptr += sprintf (outptr, "{%d+}\r\n%s", (int)strlen(string), string);
 				} else {
 					char *quoted = imap_quote_string (string);
 					
