@@ -48,6 +48,7 @@ typedef enum {
 	E_GW_ITEM_TYPE_ORGANISATION,
 	E_GW_ITEM_TYPE_RESOURCE,
 	E_GW_ITEM_TYPE_CATEGORY,
+	E_GW_ITEM_TYPE_NOTIFICATION, 
 	E_GW_ITEM_TYPE_UNKNOWN
 	
 } EGwItemType;
@@ -123,6 +124,7 @@ typedef struct {
 	char *contentType ;
 	int size ;
 	char *date ;
+	char *data ;
 } EGwItemAttachment ;
 
 typedef enum {
@@ -187,10 +189,13 @@ void 	    e_gw_item_set_to (EGwItem *item, const char *to) ;
 const char* e_gw_item_get_to (EGwItem *item) ;
 const char *e_gw_item_get_msg_content_type (EGwItem *item) ;
 guint32     e_gw_item_get_item_status (EGwItem *item);
+void	    e_gw_item_set_content_type (EGwItem *item, const char *content_type) ;
+char	    *e_gw_item_get_content_type (EGwItem *item) ;
 void e_gw_item_set_change (EGwItem *item, EGwItemChangeType change_type, char *field_name, gpointer field_value);
 gboolean e_gw_item_append_changes_to_soap_message (EGwItem *item, SoupSoapMessage *msg);
 void e_gw_item_set_category_name (EGwItem *item, char *cateogry_name);
 char* e_gw_item_get_category_name (EGwItem *item);
+void e_gw_item_set_sendoptions (EGwItem *item, gboolean set);
 void e_gw_item_set_reply_request (EGwItem *item, gboolean set);
 gboolean e_gw_item_get_reply_request (EGwItem *item);
 void e_gw_item_set_reply_within (EGwItem *item, char *reply_within);
@@ -245,11 +250,13 @@ EGwItemOrganizer *e_gw_item_get_organizer (EGwItem *item);
 void e_gw_item_set_organizer (EGwItem  *item, EGwItemOrganizer *organizer);
 
 GSList * e_gw_item_get_attach_id_list (EGwItem *item) ;
+void e_gw_item_set_attach_id_list (EGwItem *item, GSList *attach_list) ;
 
 
 GSList *e_gw_item_get_recurrence_dates (EGwItem *item);
 void e_gw_item_set_recurrence_dates (EGwItem  *item, GSList *new_recurrence_dates);
 
+void e_gw_item_set_source (EGwItem *item, char *source) ;
 
 int e_gw_item_get_trigger (EGwItem *item);
 void e_gw_item_set_trigger (EGwItem *item, int trigger);
@@ -258,6 +265,14 @@ void e_gw_item_set_trigger (EGwItem *item, int trigger);
 typedef struct {
 	char *email;
 	char *display_name;
+	gboolean status_enabled;
+	char *delivered_date;
+	char *opened_date;
+	char *accepted_date;
+	char *deleted_date;
+	char *declined_date;
+	char *completed_date;
+	char *undelivered_date;
 	enum {
 		E_GW_ITEM_RECIPIENT_TO,
 		E_GW_ITEM_RECIPIENT_CC,
